/*
	Fractal - A program to calculate Newton fractals.
	Copyright (C) 2004 Johannes Bauer
	
	This file is part of Fractal.

    Fractal is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Fractal is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Fractal; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Johannes Bauer
	Mussinanstr. 140
	92318 Neumarkt/Opf.
	JohannesBauer@gmx.de
*/

#include <stdio.h>
#include <iostream>
#include <vector>

#include "Fractal.hpp"
#include "CPoly.hpp"

CPoly::CPoly() {
}

int CPoly::CoefficientCount() {
	return this->Coefficients.size();
}

cplx CPoly::GetCoefficient(int i) {
	return this->Coefficients[i];
}

void CPoly::Display() {
	if (this->Coefficients.size() == 0) {
		std::cout << "0" << std::endl;
		return;
	}
	for (int i = this->Coefficients.size() - 1; i >= 0; i--) {
		if ((Coefficients[i].real() != 0) || (Coefficients[i].imag() != 0)) {
			std::cout << Coefficients[i];
			if (i != 0) std::cout << "*";
			if (i >= 1) std::cout << "x";
			if (i >= 2) std::cout << "^" << i;
			if (i != 0) std::cout << " + ";
		}
	}
	std::cout << std::endl;
}

void CPoly::Display_Raw() {
	if (this->Coefficients.size()==0) {
		std::cout << "0" << std::endl;
		return;
	}
	std::cout << "[";
	for (unsigned int i=0;i<Coefficients.size();i++) {
		std::cout << Coefficients[i].real() << " ";
	}
	std::cout << "]";
	std::cout << std::endl;
}

void CPoly::Add_Coefficient(cplx *Number) {
	this->Coefficients.push_back(*Number);
}

cplx* CPoly::Value(cplx *Number) {
	cplx *Sum;
	Sum = new cplx(0, 0);
	for (unsigned int i=0;i<this->Coefficients.size();i++) {
		(*Sum) += Coefficients[i]*pow((*Number),i);
	}
	return Sum;
}

void CPoly::Differentiate() {
	if (this->Coefficients.size()==0) return;
	for (unsigned int i=0;i<this->Coefficients.size()-1;i++) {
		this->Coefficients[i]=this->Coefficients[i+1]*(double)(i+1);
	}
	this->Coefficients.pop_back();
}

void CPoly::Write(FILE *f) {
	if (this->Coefficients.size()==0) return;
	for (unsigned int i=0;i<this->Coefficients.size();i++) {
		fprintf(f,"%.46f",this->Coefficients[i].real());
		if (i!=(this->Coefficients.size()-1)) fprintf(f," ");
	}
	fprintf(f,"\n");
}

CPoly::~CPoly() {
}
